#!/bin/bash

repo_root="/vagrant"
config_file="rspec.json"

usage() {
    echo "Usage: $0 create|start|stop|destroy [-f config_file] [-r repo_root] [-c container_name|all]" 1>&2
    echo "    -f defaults to '${config_file}'" 1>&2
    echo "    -r defaults to '${repo_root}'" 1>&2
    echo "    -c must be specified, use 'all' for all" 1>&2
    exit 1
}

create() {
    service_name=$1
    config_file=$2
    repo_root=$3
    if ! lxc list | grep ${service_name}; then
        # create a container with a static ip address
        echo "Creating container: ${service_name}"
        SERVICE=$(jq --arg NAME ${service_name} '.[] | select(.name==$NAME)' ${config_file})
        echo $SERVICE
        ip=$(echo $SERVICE | jq -r '.ip_address')

        lxc init ubuntu:16.04 ${service_name}
        echo "Creating network: ${service_name}"
        lxc network attach lxcbr0 ${service_name} eth0
        lxc config device set ${service_name} eth0 ipv4.address ${ip}
        lxc config set ${service_name} security.privileged true

        # copy flame clmc files into the root container
        echo "Copying files to rootfs"
        container_dir="/var/lib/lxd/containers/"${service_name}"/rootfs"
        container_vagrant_dir=${container_dir}"/vagrant"
        mkdir -p ${container_vagrant_dir}       
        cp -f ${repo_root}/reporc ${container_vagrant_dir}   
        cp -rf ${repo_root}/scripts ${container_vagrant_dir}     
        cp -rf ${repo_root}/src ${container_vagrant_dir}
        chown -R 100000:100000 ${container_vagrant_dir} 

        # start the container
        echo "Starting: ${service_name}"
        lxc start ${service_name}
        while :; do
            echo "Waiting for container to start: ${service_name}"
            lxc file pull ${service_name}/etc/resolv.conf - | grep -q nameserver && break
            sleep 1
        done

        # provision software into each container
        echo "Provisioning: ${service_name}"
        if [ ${service_name} == "clmc-service" ]; then
            influxdb_url=$(echo $SERVICE | jq -r '.influxdb_url')
            database_name=$(echo $SERVICE | jq -r '.database_name')
            report_period=$(echo $SERVICE | jq -r '.report_period')
            cmd="/vagrant/scripts/clmc-service/install.sh ${influxdb_url} ${database_name} ${report_period}"
            echo "Provisioning command ${cmd}"
            lxc exec ${service_name} --env REPO_ROOT="/vagrant" -- ${cmd}
            exit_code=$?
            if [ $exit_code != 0 ]; then
                echo "clmc-service installation failed with exit code ${exit_code}"
                exit 1
            fi            
        elif [ ${service_name} == "test-runner" ]
        then
            cmd=/vagrant/src/test/clmctest/services/pytest/install.sh
            lxc exec ${service_name} -- ${cmd}
            exit_code=$?
            if [ $exit_code != 0 ]; then
                echo "test-runner installation failed with exit code ${exit_code}"
                exit 1
            fi            
        else
            # get container parameters
            location=$(echo $SERVICE | jq -r '.location')
            sf_id=$(echo $SERVICE | jq -r '.sf_id')
            sf_id_instance=$(echo $SERVICE | jq -r '.sf_id_instance')
            sfc_id=$(echo $SERVICE | jq -r '.sfc_id')
            sfc_id_instance=$(echo $SERVICE | jq -r '.sfc_id_instance')
            sr_id=$(echo $SERVICE | jq -r '.sr_id')
            ipendpoint_id=$(echo $SERVICE | jq -r '.ipendpoint_id')
            influxdb_url=$(echo $SERVICE | jq -r '.influxdb_url')
            database_name=$(echo $SERVICE | jq -r '.database_name')

            # install service function specific software
            cmd=/vagrant/src/test/clmctest/services/${sf_id}/install.sh
            lxc exec ${service_name} --env REPO_ROOT="/vagrant" -- ${cmd}
            exit_code=$?
            if [ $exit_code != 0 ]; then
                echo "${sf_id} installation failed with exit code ${exit_code}"
                exit 1
            fi
            # install telegraf
            cmd=/vagrant/scripts/clmc-agent/install.sh
            lxc exec ${service_name} --env REPO_ROOT="/vagrant" -- /vagrant/scripts/clmc-agent/install.sh

            # check that telegraf installed (it may not have if the reporc file was not present or Nexus server was down)
            if lxc-attach -n ${service_name} -- ls /etc/telegraf |& grep 'ls: cannot access'; then
                echo "Telegraf agent failed to install (check reporc?)"
                exit 1
            fi

            # stop telegraf before changing the configs
            lxc exec ${service_name} -- service telegraf stop

            # copy telegraf configuration templates
            cp -f ${repo_root}/scripts/clmc-agent/telegraf.conf ${container_dir}/etc/telegraf/
            cp -f ${repo_root}/scripts/clmc-agent/telegraf_output.conf ${container_dir}/etc/telegraf/telegraf.d/
            # copy the 'host' config into all service containers
            cp -f ${repo_root}/src/test/clmctest/services/host/telegraf*.conf ${container_dir}/etc/telegraf/telegraf.d/
            # copy the service-specific config
            cp -f ${repo_root}/src/test/clmctest/services/${sf_id}/telegraf*.conf ${container_dir}/etc/telegraf/telegraf.d/
            chown -R 100000:100000 ${container_dir}/etc/telegraf/

            # replace telegraf template with container parameters
            cmd="/vagrant/scripts/clmc-agent/configure.sh ${location} ${sfc_id} ${sfc_id_instance} ${sf_id} ${sf_id_instance} ${ipendpoint_id} ${sr_id} ${influxdb_url} ${database_name}"
            lxc exec ${service_name} -- ${cmd}

            # start telegraf
            lxc exec ${service_name} -- service telegraf start
        fi

        # set forward ports
        ports=$(echo $SERVICE | jq -r '.forward_ports')
        if [ "$ports" != "null" ]; then
            for row in $(echo "${ports}" | jq -r '.[] | @base64'); do
                _jq() {
                echo ${row} | base64 --decode | jq -r ${1}
                }
                guest_port=$(_jq '.guest')
                host_port=$(_jq '.host')
                iptables -t nat -I PREROUTING -i enp0s3 -p TCP -d 10.0.2.15 --dport ${host_port} -j DNAT --to-destination ${ip}:${guest_port}
            done
        fi
    fi
}

start() {
    service_name=$1
    if lxc info ${service_name}; then
        echo "Starting container: ${service_name}"
        lxc start ${service_name}
    fi
}

stop() {
    service_name=$1
    if lxc info ${service_name}; then
        echo "Stopping container: ${service_name}"
        lxc stop -n ${service_name}
    fi
}

destroy() {
    service_name=$1
    config_file=$2
    if lxc list | grep ${service_name}; then
        echo "Stopping container: ${service_name}"
        lxc stop ${service_name}
        echo "Destroying container: ${service_name}"
        lxc delete ${service_name}

        # remove forward ports
        ports=$(echo $SERVICE | jq -r '.forward_ports')
        if [ "$ports" != "null" ]; then
            echo "destroy ports"
            for row in $(echo "${ports}" | jq -r '.[] | @base64'); do
                _jq() {
                echo ${row} | base64 --decode | jq -r ${1}
                }
                guest_port=$(_jq '.guest')
                host_port=$(_jq '.host')
                iptables -t nat -D PREROUTING -i enp0s3 -p TCP -d 10.0.2.15 --dport ${host_port} -j DNAT --to-destination ${ip}:${guest_port}
            done
        fi
    fi
}

# inc option index by 1 as first argument is the command and not parsed by getopts
if [ $# -gt 1 ]; then
    OPTIND=$((OPTIND+1))
fi

# collect the optional arguments
while getopts "hf:r:c:" opt; do
  case $opt in
    h) usage; exit ;;
    f) config_file=${OPTARG} ;;
    r) repo_root=${OPTARG} ;;
    c) container=${OPTARG} ;;
    \?) usage ;;
  esac
done

# check configuration file exists
if [ ! -f ${config_file} ]; then
    echo "Configuration file does not exist: ${config_file}" >&2
    usage
    exit 1
fi

# check repo root directory exists
if [ ! -d ${repo_root} ]; then
    echo "Repo root directory does not exist: ${repo_root}" >&2
    usage
    exit 1
fi

# check a service has been defined
if [ -z ${container} ]; then
    echo "A container must be named: use 'all' for all"
    usage
    exit 1
fi

# iterate of list of services in configuration file
command=$1
service_names=$(jq -r '.[].name' ${config_file})
for service_name in $service_names; do
    # if all or specific container
    if [ ${container} = all -o ${container} = ${service_name} ]; then
        case "${command}" in
            create)
                create ${service_name} ${config_file} ${repo_root}
                ;;
            start)
                start ${service_name} ${config_file} ${repo_root}
                ;;
            stop)
                stop ${service_name} ${config_file} ${repo_root}
                ;;
            destroy)
                destroy ${service_name} ${config_file} ${repo_root}
                ;;
            *)
                usage
                ;;
        esac
    fi
done



echo -e "\n\n------>Create iptables summary"
iptables -t nat -L -n -v
iptables-save > /etc/iptables/rules.v4
